Imports Microsoft.VisualBasic
Imports System
Imports System.Collections.Generic
Imports System.Linq
Imports System.Text
Imports System.Threading
Imports System.Workflow.Runtime
Imports System.Workflow.Runtime.Hosting

Namespace ErrorHandlingInWF
	Friend Class Program
		Private Shared workflowRuntime As WorkflowRuntime
        Private Shared waitHandle As New AutoResetEvent(False)

		Shared Sub Main(ByVal args() As String)
			workflowRuntime = New WorkflowRuntime()
			Using workflowRuntime

				AddHandler workflowRuntime.WorkflowCompleted, Function(ByVal sender As Object, ByVal e As WorkflowCompletedEventArgs) waitHandle.Set()

				'capture terminated workflows and get notified of the exception
                AddHandler workflowRuntime.WorkflowTerminated, AddressOf WorkflowTerminated

				'Get notified of exceptions occuring in the runtime services
                AddHandler workflowRuntime.ServicesExceptionNotHandled, AddressOf ServicesExceptionNotHandled

				Dim actions As Dictionary(Of String, Action) = New Dictionary(Of String, Action)()
				actions.Add("Manual Retry", AddressOf RunManualRetry)
				actions.Add("Auto Retry", AddressOf RunRetry)
				actions.Add("Error handling", AddressOf RunErrorHandling)

				Dim menu As New ConsoleMenu(actions)
				menu.Run()


				waitHandle.WaitOne()
			End Using


        End Sub

        Private Shared Sub ServicesExceptionNotHandled(ByVal sender As Object, ByVal snhe As ServicesExceptionNotHandledEventArgs)
            Console.WriteLine(snhe.Exception.Message)
        End Sub


        Private Shared Sub WorkflowTerminated(ByVal sender As Object, ByVal e As WorkflowTerminatedEventArgs)
            Console.WriteLine(e.Exception.Message)
            WaitHandle.Set()
        End Sub
		Private Shared Sub RunManualRetry()
			Dim instance As WorkflowInstance = workflowRuntime.CreateWorkflow(GetType(ErrorHandlingInWF.ManualRetryLogic))
			instance.Start()
		End Sub

		Private Shared Sub RunRetry()
			Dim instance As WorkflowInstance = workflowRuntime.CreateWorkflow(GetType(ErrorHandlingInWF.WorkflowWithRetryActivity))
			instance.Start()
		End Sub

		Private Shared Sub RunErrorHandling()
			Dim instance As WorkflowInstance = workflowRuntime.CreateWorkflow(GetType(ErrorHandlingInWF.Workflow1))
			instance.Start()
		End Sub

	End Class
End Namespace
